﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HeuristicLab.Problems.LinearOrdering
{ /// <summary>
  /// An operator which evaluates LOP solutions given in matrix representation.
  /// </summary>
    [Item("AwesomeTriangulationEvaluator", "An operator which evaluates LOP solutions given in matrix representation.")]
    [StorableClass]
    public sealed class AwesomeTriangulationEvaluator : MatrixTriangulationEvaluator
    {
        [StorableConstructor]
        private AwesomeTriangulationEvaluator(bool deserializing) : base(deserializing) { }
        private AwesomeTriangulationEvaluator(AwesomeTriangulationEvaluator original, Cloner cloner) : base(original, cloner) { }
        public AwesomeTriangulationEvaluator() : base() { }

        public override IDeepCloneable Clone(Cloner cloner)
        {
            return new AwesomeTriangulationEvaluator(this, cloner);
        }

        /// <summary>
        /// Calculates the distance between two points using the rounded Euclidean distance metric.
        /// </summary>
        /// <param name="x1">The x-coordinate of point 1.</param>
        /// <param name="y1">The y-coordinate of point 1.</param>
        /// <param name="x2">The x-coordinate of point 2.</param>
        /// <param name="y2">The y-coordinate of point 2.</param>
        /// <returns>The calculated distance.</returns>
        protected override double CalculateUpperTriangle(DoubleMatrix m, Permutation p)
        {
            int[] permutation = p.ToArray();

            //reduce offset, real arrays start with 0
            if (!permutation.Contains(0)) { permutation = permutation.Select(v => v - 1).ToArray(); }

            double sum = 0;
            for (int i = 1; i < m.Columns; i++)
            {
                for (int j = 0; j < i; j++)
                {
                    sum += m[p[j], p[i]];
                }
            }

            return sum;
        }
    }
}
