#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.LinearOrdering
{
    /// <summary>
    /// A base class for operators which evaluate LOP solutions.
    /// </summary>
    [Item("LOPEvaluator", "A base class for operators which evaluate LOP solutions.")]
    [StorableClass]
    public abstract class LOPEvaluator : InstrumentedOperator, ILOPEvaluator
    {
        public override bool CanChangeName {
            get { return false; }
        }

        public ILookupParameter<DoubleValue> QualityParameter {
            get { return (ILookupParameter<DoubleValue>)Parameters["Quality"]; }
        }

        [StorableConstructor]
        protected LOPEvaluator(bool deserializing) : base(deserializing) { }
        protected LOPEvaluator(LOPEvaluator original, Cloner cloner) : base(original, cloner) { }
        protected LOPEvaluator()
          : base()
        {
            Parameters.Add(new LookupParameter<DoubleValue>("Quality", "The evaluated quality of the LOP solution."));
        }
    }
}
